from maggma.builder import Builder

__author__ = "Shyam Dwaraknath"
__email__ = "shyamd@lbl.gov"

class TaskTagger(Builder):
    def __init__(self, tasks, task_types, **kwargs):
        """
        Creates task_types from tasks and type definitions

        Args:
            tasks (Store): Store of task documents
            task_defs (Store): Store of task_definitions. These define 
            task_types (Store): Store of task_types for tasks
        """
        self.tasks = tasks
        self.task_types = task_types

        super().__init__(sources=[tasks],
                         targets=[task_types],
                         **kwargs)

    def get_items(self):
        """
        Returns all task docs and tag definitions to process

        Returns:
            generator or list of task docs and tag definitions
        """

        # Get all tasks ids from the tasks collection
        all_task_ids = self.tasks.collection.distinct("task_id", {"state": "successful"})

        
        # Figure out which task_ids are not in the materials collection and only process those
        previous_task_ids = self.task_types.collection.distinct("task_id", {"task_type": {"$exists": 0}})
        to_process = set(all_task_ids) - set(previous_task_ids)

        # Process each task_id
        for t_id in to_process:
            print("Processing task_id: {}".format(t_id))
            try:
                yield self.tasks.collection.find_one({"task_id": t_id})
            except:
                import traceback
                print("Problem processing task_id: {}".format(t_id))

    def process_item(self, item):
        """
        Find the task_type for the item 
        
        Args:
            item ((dict,[dict])): a task doc and a list of possible tag definitions
        """

        return {"task_id": item["task_id"],
                "task_type": self.task_type(item)}

    def update_targets(self, items):
        """
        Inserts the new task_types into the task_types collection
        
        Args:
            items ([dict]): task_type dicts to insert into task_types collection
        """
        for doc in items:
            self.task_types.collection.update({'task_id': doc['task_id']}, doc, upsert=True)

    def finalize(self):
        pass

    def task_type(self, task_doc):
        """
        Determines the task_type

        Args:
            task_doc (dict): task_document with original input
        """
        incar = task_doc["input_orig"]["INCAR"]

        if incar.get("LHFCALC",False):
            if incar.get("NSW") == 0:
                return "hse bs"
            else:
                return "hse"

        if incar.get("ICHARG",0) > 10:
            if incar.get("NEDOS",0) > 600:
                return "nscf uniform"
            else:
                return "nscf line"


        if incar.get("LEPSILON",False):
            return "static dielectric"


        if incar.get("IBRION",0) < 0:
            return "static"


        if incar.get("ISIF",2) == 3 and incar.get("IBRION",0) > 0:
            return "structure optimization"

        return ""


__author__ = "Joseph Montoya"
__email__ = "montoyjh@lbl.gov"

class MPSubmissionFilter(object):
    """
    Minimal filter that takes an SNL, checks against 
    a database of previous submissions, optionally
    checks against a materials collection, generates
    a set of production workflows, adds them to a
    launchpad
    """

    # Set of default matching params
    default_match_params = {}
    # Set of default properties-workflow
    default_property_wflows = {"bandstructure": "wf_bandstructure",
                               "elasticity": "wf_elastic_constant",
                               "piezo": "wf_piezoelectric_constant"}

    def __init__(self, submission_coll, materials_coll=None):
        """
        submissions_coll (collection):
        materials_coll (collection):
        """
        self.submission_coll = submission_coll
        self.materials_coll = materials_coll

    @staticmethod
    def find_duplicate_snl(snl, collection, match_params=None):
        """
        Args:
            structure (Structure): struct object to attempt
                finding in the submissions db
            match_params (dict): dictionary of parameters
                for structure matcher
        """
        match_params = match_params or default_match_params
        sm = StructureMatcher(**match_params)
        formula_matches = collection.find({
            "pretty_formula": snl.composition.reduced_formula},
            {"snl": 1, "submission_id": 1})
        for formula_match in formula_matches:
            prev_snl = formula_match["snl"]
            if sm.fit(prev_snl, snl):
                return formula_match
        return None

    def add_submission(self, snl):
        """
        Submission document is minimal, simply an SNL and formula
        """
        snl_doc = {"snl":snl.as_dict(), 
                   "pretty_formula": snl.composition.reduced_formula}
        return self.submissions_coll.insert(snl_doc)

    def update_submission(self, prev_submission_id, new_snl):
        """
        """
        self.submissions_coll.update({"submission_id": prev_submission_id},
                                     {"$push": {"submitted": new_snl}})

    def get_wflows(snl, properties=None, materials_coll=None):
        """
        Args:
            properties (dict): dictionary such that 
        """

    def submit(self, snl, dupecheck=True, wflows=None,
               match_params=None, launchpad=None):
        """
        TODO: Add this docstring
        TODO: prevent race condition?
        """
        dup_submission = self.find_duplicate(snl, self.submission_coll,
                                             match_params=match_params)
        if dup_submission:
            # add duplicate to submissions db?
            # self.update_submission(dup_submission["submission_id"], snl)
            # logger print duplicate submission
            return None
        
        if self.materials_coll:
            dup_material = self.find_duplicate(snl, self.materials_coll,
                                               match_params=match_params)
            if dup_material:
                # add material snl to submissions collection
                # logger print duplicate submission from materials coll
                return None
        
        # Check for properties specific to builder?
        wflows = get_wflows(snl)





#!/usr/bin/env python

import os

from setuptools import setup, find_packages

module_dir = os.path.dirname(os.path.abspath(__file__))

if __name__ == "__main__":
    setup(
        name='emmet',
        version='0.1',
        description='Emmet is a builder framework for the Materials Project',
        long_description=open(os.path.join(module_dir, 'README.md')).read(),
        url='https://github.com/materialsproject/emmet',
        author='MP team',
        author_email='matproj-develop@googlegroups.com',
        license='modified BSD',
        packages=find_packages(),
        package_data={},
        zip_safe=False,
        # TODO: finalize requirements
        install_requires=[
            'FireWorks>=1.4.0', 'pymatgen>=4.7.1', 'pymatgen-db>=0.5.1',
            'maggma', 'monty>=0.9.5', 'six',
        ],
        classifiers=["Programming Language :: Python :: 3",
                     "Programming Language :: Python :: 3.6",
                     'Development Status :: 2 - Pre-Alpha',
                     'Intended Audience :: Science/Research',
                     'Intended Audience :: System Administrators',
                     'Intended Audience :: Information Technology',
                     'Operating System :: OS Independent',
                     'Topic :: Other/Nonlisted Topic',
                     'Topic :: Scientific/Engineering'],
        test_suite='nose.collector',
        tests_require=['nose']
    )
